package com.yihu.wlyy.service.weixin.wxpay.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yihu.wlyy.entity.organization.HospitalMapping;
import com.yihu.wlyy.repository.organization.HospitalMappingDao;
import com.yihu.wlyy.service.third.httplog.LogService;
import com.yihu.wlyy.service.weixin.wxpay.common.Configure;
import com.yihu.wlyy.service.weixin.wxpay.model.BindCard;
import com.yihu.wlyy.service.weixin.wxpay.model.Charge;
import com.ylzinfo.onepay.sdk.OnepayDefaultClient;
import com.ylzinfo.onepay.sdk.domain.RequestParams;
import com.ylzinfo.onepay.sdk.domain.ResponseParams;
import com.ylzinfo.onepay.sdk.utils.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.PrintWriter;
import java.io.StringWriter;

/**
 * 易联众统一支付服务类
 * add by hzp at 20170523
 */
@Service
public class OnePayService {

    @Autowired
    private Configure config;

    @Autowired
    private LogService logService;

    private String signType= "MD5";
    private String encryptType= "AES";

    @Autowired
    private HospitalMappingDao hospitalMappingDao;

    /**
     * 查询绑卡信息
     */
    public BindCard bindCard(String openid) throws Exception
    {
        Boolean isSuccess = true;
        String msgBody = "";
        String response = "";
        String error = "";
        BindCard result = null;

        String appId = config.getOnepayAppId();
        String appSecret = config.getOnepayAppSecret();
        OnepayDefaultClient onepayClient = new OnepayDefaultClient(config.getOnepayApi(),appId, appSecret, signType, encryptType);

        try {
            RequestParams requestParams = new RequestParams();
            requestParams.setAppId(appId);
            requestParams.setTimestamp(DateUtil.getCurrentDateTime());
            requestParams.setSignType(signType);
            requestParams.setEncryptType(encryptType);
            requestParams.setTimestamp(DateUtil.getCurrentDateTime());

            // 请求报文体参数，详见接口文档各接口——请求参数定义
            // 如医保绑卡信息请求参数
            JSONObject params = new JSONObject();
            params.put("userId", openid);
            requestParams.setParam(params);
            requestParams.setTransType(config.getBindCardType());
            ResponseParams<JSONObject> res = onepayClient.execute(requestParams);

            msgBody =  JSON.toJSONString(requestParams);
            response =  JSON.toJSONString(res);

            // 如医保卡绑卡信息返回参数
            if (OnepayDefaultClient.isSuccessful(res)) {
                result = JSON.toJavaObject(res.getParam(), BindCard.class);

                //业务处理***********
                System.out.println("请求成功,返回参数:" + result.getCardNo() + "," +  result.getIdType() + "," + result.getIdNo() + "," + result.getUserName() + "," + result.getUserId());

            }
            else {
                isSuccess = false;
                error = "请求失败,返回结果:" + res.getRespCode() + "," + res.getRespMsg();
            }
        }
        catch (Exception ex) {
            isSuccess = false;
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            ex.printStackTrace(pw);
            error = sw.toString();
        }

        //保存http日志
        logService.saveHttpLog(isSuccess,config.getBindCardType(),"查询绑卡信息","POST",null,msgBody,response,error,logService.onepayType);

        if(!isSuccess)
        {
            throw new Exception(error);
        }

        return result;
    }

    /**
     * 家庭医生签约支付
     */
    public String charge(String orgCode,String code,String date,String cardNo,String openid) throws Exception
    {
        //获取机构映射
        HospitalMapping hos = hospitalMappingDao.findByCode(orgCode);
        if(hos==null)
        {
            throw new Exception("no hospital mapping!");
        }

        return charge(hos.getAppId(), hos.getAppSecret(), code, date, cardNo, openid);
    }

    /**
     * 家庭医生签约支付
     */
    public String charge(String appId,String appSecret,String code,String date,String cardNo,String openid) throws Exception
    {
        Boolean isSuccess = true;
        String msgBody = "";
        String response = "";
        String error = "";
        String result = "";

        OnepayDefaultClient onepayClient = new OnepayDefaultClient(config.getOnepayApi(),appId, appSecret, signType, encryptType);
        try {
            RequestParams requestParams = new RequestParams();
            requestParams.setAppId(appId);
            requestParams.setTimestamp(DateUtil.getCurrentDateTime());
            requestParams.setSignType(signType);
            requestParams.setEncryptType(encryptType);
            requestParams.setTimestamp(DateUtil.getCurrentDateTime());
            requestParams.setTransType(config.getChargeType());

            //业务参数
            JSONObject params = new JSONObject();

            params.put("outSettleNo",code);  //接入应用结算业务流水号
            params.put("outSettleTime",date);    //接入应用内结算时间
            params.put("userId",openid);   //用户 ID
            params.put("cardNo",cardNo);   //医保卡号"D64775061"
            params.put("totalAmount","1");   //交易总金额
            params.put("selfpayAmount","0");   //自费金额
            params.put("insuranceAmount","1");   //医保支付金额
            params.put("returnUrl","http://");   //页面跳转地址
            params.put("feeDetail","{\"itemName\":\"家庭医生签约支付\",\"itemDesc\":\"家庭医生签约支付\",\"itemOrigPrice\":\"120\",\"itemNowPrice\":\"120\",\"itemNum\":\"1\",\"itemTotalAmt\":\"120\"}");   //费用明细
            params.put("channel","WX_MMP");  //微信医保支付
            //extendParams 扩展入参
            //requestContent 请求结构体

            requestParams.setParam(params);
            //执行支付
            ResponseParams<JSONObject> res = onepayClient.execute(requestParams);
            msgBody =  JSON.toJSONString(requestParams);
            response =  JSON.toJSONString(res);

            if (OnepayDefaultClient.isSuccessful(res)) {
                JSONObject json = res.getParam();
                result = String.valueOf(json.get("settleNo"));      //医保结算业务流水号
                String payUrl =  String.valueOf(json.get("payUrl"));

                //业务处理***********

            } else {
                isSuccess = false;
                error = "请求失败,返回结果:" + res.getRespCode() + "," + res.getRespMsg();
            }
        } catch (Exception ex) {
            isSuccess = false;
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            ex.printStackTrace(pw);
            error = sw.toString();
        }


        //保存http日志
        logService.saveHttpLog(isSuccess,config.getChargeType(),"家庭医生签约支付","POST",null,msgBody,response,error,logService.onepayType);

        if(!isSuccess)
        {
            throw new Exception(error);
        }

        return result;
    }

    /**
     * 家庭医生签约支付查询
     */
    public Charge chargeQuery(String code) throws Exception
    {
        Boolean isSuccess = true;
        String msgBody = "";
        String response = "";
        String error = "";
        Charge result = null;

        String appId = config.getOnepayAppId();
        String appSecret = config.getOnepayAppSecret();
        OnepayDefaultClient onepayClient = new OnepayDefaultClient(config.getOnepayApi(),appId, appSecret, signType, encryptType);
        try {
            RequestParams requestParams = new RequestParams();
            requestParams.setAppId(appId);
            requestParams.setTimestamp(DateUtil.getCurrentDateTime());
            requestParams.setSignType(signType);
            requestParams.setEncryptType(encryptType);
            requestParams.setTimestamp(DateUtil.getCurrentDateTime());
            requestParams.setTransType(config.getChargeQueryType());

            //业务参数
            JSONObject params = new JSONObject();
            params.put("outSettleNo",code);  //接入应用结算业务流水号
            params.put("settleNo","");
            requestParams.setParam(params);
            //执行支付
            ResponseParams<JSONObject> res = onepayClient.execute(requestParams);
            msgBody =  JSON.toJSONString(requestParams);
            response =  JSON.toJSONString(res);

            if (OnepayDefaultClient.isSuccessful(res)) {
                result = JSON.toJavaObject(res.getParam(),Charge.class);

                //业务处理*******************
                System.out.println("请求成功,返回参数:" + result.getTradeStatus() +"  " + result.getUserName());
            } else {
                isSuccess = false;
                error = "请求失败,返回结果:" + res.getRespCode() + "," + res.getRespMsg();
            }
        }
        catch (Exception ex) {
            isSuccess = false;
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            ex.printStackTrace(pw);
            error = sw.toString();
        }

        //保存http日志
        logService.saveHttpLog(isSuccess,config.getChargeType(),"家庭医生签约支付","POST",null,msgBody,response,error,logService.onepayType);

        if(!isSuccess)
        {
            throw new Exception(error);
        }

        return result;
    }
}
